@@ -13,20 +13,23 @@ from sql_to_ibis.exceptions.sql_exception import (
     InvalidQueryException,
     TableExprDoesNotExist,
 )
-from sql_to_ibis.parsing.transformers import InternalTransformer, TransformerBaseClass
+from sql_to_ibis.parsing.transformers import (
+    InternalTransformer,
+    InternalTransformerWithStarVal,
+    TransformerBaseClass,
+)
 from sql_to_ibis.query_info import QueryInfo
 from sql_to_ibis.sql_objects import (
     Aggregate,
     AliasRegistry,
     AmbiguousColumn,
     Column,
+    CountStar,
     CrossJoin,
     DerivedColumn,
-    Expression,
     GroupByColumn,
     Join,
     JoinBase,
-    Literal,
     Subquery,
     Table,
     Value,
@@ -178,12 +181,7 @@ class SQLTransformer(TransformerBaseClass):
         return query_info
 
     def _handle_join_subqueries(self, join: JoinBase) -> QueryInfo:
-        info = QueryInfo(
-            having_expr=None,
-            where_expr=None,
-            internal_transformer=InternalTransformer.empty_transformer(),
-            distinct=False,
-        )
+        info = QueryInfo(InternalTransformer.empty_transformer(),)
         info.add_table(join)
         info.add_column(Column(name="*"))
         return info
@@ -201,7 +199,7 @@ class SQLTransformer(TransformerBaseClass):
             query_info = self._handle_join_subqueries(query_object)
         else:
             query_info = query_object
-        subquery_value = self.to_ibis_table(query_info)
+        subquery_value = self._to_ibis_table(query_info)
         subquery = Subquery(name=alias_name, value=subquery_value)
         self._table_map[alias_name] = subquery
         self._column_name_map[alias_name] = {}
@@ -323,46 +321,6 @@ class SQLTransformer(TransformerBaseClass):
                 return True
         return False
 
-    @staticmethod
-    def handle_non_token_non_tree(query_info: QueryInfo, token, token_pos):
-        """
-        Handles non token_or_tree non tree items and extracts necessary query
-        information from it
-
-        :param query_info: Dictionary of all info about the query
-        :param token: Item being handled
-        :param token_pos: Ordinal position of the item
-        :return:
-        """
-        query_info.all_names.append(token.final_name)
-        query_info.name_order[token.final_name] = token_pos
-
-        if isinstance(token, GroupByColumn):
-            query_info.group_columns.append(token)
-        elif isinstance(token, (Column, Literal, Expression)):
-            query_info.add_column(token)
-        elif isinstance(token, Aggregate):
-            query_info.aggregates[token.final_name] = token
-
-    def handle_token_or_tree(self, query_info: QueryInfo, token_or_tree, item_pos):
-        """
-        Handles token and extracts necessary query information from it
-        :param query_info: Dictionary of all info about the query
-        :param token_or_tree: Item being handled
-        :param item_pos: Ordinal position of the token
-        :return:
-        """
-        if isinstance(token_or_tree, Token):
-            if token_or_tree.type == "from_expression":
-                query_info.add_table(token_or_tree.value)
-            elif token_or_tree.type == "where_expr":
-                query_info.where_expr = token_or_tree.value
-        elif isinstance(token_or_tree, Tree):
-            if token_or_tree.data == "having_expr":
-                query_info.having_expr = token_or_tree
-        else:
-            self.handle_non_token_non_tree(query_info, token_or_tree, item_pos)
-
     def select(self, *select_expressions: Tuple[Tree]) -> QueryInfo:
         """
         Forms the final sequence of methods that will be executed
@@ -397,13 +355,6 @@ class SQLTransformer(TransformerBaseClass):
                 elif select_expression.data == "where_expr":
                     where_expr = select_expression
 
-        select_expressions_no_boolean_clauses = tuple(
-            select_expression
-            for select_expression in select_expressions
-            if isinstance(select_expression, Tree)
-            and select_expression.data not in ("having_expr", "where_expr")
-            or not isinstance(select_expression, Tree)
-        )
         internal_transformer = InternalTransformer(
             tables,
             self._table_map,
@@ -413,22 +364,20 @@ class SQLTransformer(TransformerBaseClass):
             self._alias_registry,
         )
 
-        select_expressions = internal_transformer.transform(
-            Tree("select", select_expressions_no_boolean_clauses)
-        ).children
+        select_expressions_no_boolean_clauses = tuple(
+            select_expression
+            for select_expression in select_expressions
+            if isinstance(select_expression, Tree)
+            and select_expression.data not in ("having_expr", "where_expr")
+            or not isinstance(select_expression, Tree)
+        )
 
-        distinct = False
-        if isinstance(select_expressions[0], Token):
-            if str(select_expressions[0]) == "distinct":
-                distinct = True
-            select_expressions = select_expressions[1:]
-
-        query_info = QueryInfo(having_expr, where_expr, internal_transformer, distinct)
-
-        for token_pos, token in enumerate(select_expressions):
-            self.handle_token_or_tree(query_info, token, token_pos)
-
-        return query_info
+        return QueryInfo(
+            internal_transformer,
+            select_expressions_no_boolean_clauses,
+            having_expr=having_expr,
+            where_expr=where_expr,
+        )
 
     def cross_join(self, table1: Table, table2: Table):
         """
@@ -446,10 +395,19 @@ class SQLTransformer(TransformerBaseClass):
     def format_column_needs_agg_or_group_msg(column):
         return f"For column '{column}' you must either group or provide an aggregation"
 
-    def _get_aggregate_ibis_columns(self, aggregates: Dict[str, Aggregate]):
+    def _handle_count_star(self, aggregate: Aggregate, relation: TableExpr):
+        if isinstance(aggregate.value, CountStar):
+            aggregate.value = relation.count()
+        return aggregate
+
+    def _get_aggregate_ibis_columns(
+        self, aggregates: Dict[str, Aggregate], relation: TableExpr
+    ):
         aggregate_ibis_columns = []
-        for aggregate_column in aggregates:
-            column = aggregates[aggregate_column].value.name(aggregate_column)
+        for aggregate_column_name in aggregates:
+            aggregate_column = aggregates[aggregate_column_name]
+            self._handle_count_star(aggregate_column, relation)
+            column = aggregate_column.value.name(aggregate_column_name)
             aggregate_ibis_columns.append(column)
         return aggregate_ibis_columns
 
@@ -488,7 +446,7 @@ class SQLTransformer(TransformerBaseClass):
         selected_column_names = {
             column.get_name().lower() for column in selected_columns
         }
-        aggregate_ibis_columns = self._get_aggregate_ibis_columns(aggregates)
+        aggregate_ibis_columns = self._get_aggregate_ibis_columns(aggregates, table)
         having = self._handle_having_expressions(
             having_expr,
             internal_transformer,
@@ -673,13 +631,12 @@ class SQLTransformer(TransformerBaseClass):
             return result[all_columns]
         return result
 
-    def _set_casing_for_groupby_names(
-        self, groupby_columns: List[GroupByColumn], selected_columns: List[Value]
-    ):
+    def _set_casing_for_groupby_names(self, query_info: QueryInfo):
         lower_case_to_true_column_name = {
-            column.get_name().lower(): column.get_name() for column in selected_columns
+            column.get_name().lower(): column.get_name()
+            for column in query_info.columns
         }
-        for groupby_column in groupby_columns:
+        for groupby_column in query_info.group_columns:
             lower_case_group_name = groupby_column.get_name().lower()
             if lower_case_group_name in lower_case_to_true_column_name:
                 selection_statement_name = lower_case_to_true_column_name[
@@ -697,16 +654,27 @@ class SQLTransformer(TransformerBaseClass):
         if isinstance(table, JoinBase):
             return table
 
-    def to_ibis_table(self, query_info: QueryInfo):
-        """
-        Returns the dataframe resulting from the SQL query
-        :return:
-        """
+    @staticmethod
+    def _reevaluate_transformation(query_info: QueryInfo, relations: List[TableExpr]):
+        DerivedColumn.reset_expression_count()
+        internal_transformer = InternalTransformerWithStarVal.from_internal_transformer(
+            internal_transformer=query_info.internal_transformer,
+            available_relations=relations,
+        )
+        query_info = QueryInfo(
+            internal_transformer,
+            query_info.select_expressions_no_boolean_clauses,
+            query_info.having_expr,
+            query_info.where_expr,
+        )
+        query_info.perform_transformation()
+        return query_info
+
+    def _get_relation(self, query_info: QueryInfo):
         tables = query_info.tables
-        if not query_info.tables:
+        if not tables:
             raise Exception("No table specified")
         first_table = self.get_table_value(tables[0])
-
         if isinstance(first_table, JoinBase):
             first_table = self.handle_join(join=first_table, columns=query_info.columns)
         for table in tables[1:]:
@@ -717,8 +685,19 @@ class SQLTransformer(TransformerBaseClass):
                 [table for table in tables if isinstance(table, Table)]
             )
             first_table = first_table[all_columns]
+        return first_table
 
-        self._set_casing_for_groupby_names(query_info.group_columns, query_info.columns)
+    def _to_ibis_table(self, query_info: QueryInfo):
+        """
+        Returns the dataframe resulting from the SQL query
+        :return:
+        """
+        query_info.perform_transformation()
+
+        relation = self._get_relation(query_info)
+        print(relation)
+
+        self._set_casing_for_groupby_names(query_info)
 
         selected_group_columns = []
         group_column_names = {
@@ -744,7 +723,7 @@ class SQLTransformer(TransformerBaseClass):
                 for column in query_info.columns
                 if column.get_name() in remaining_columns
             ]
-        new_table = self.handle_selection(first_table, query_info.columns)
+        new_table = self.handle_selection(relation, query_info.columns)
         new_table = self.handle_filtering(
             new_table, query_info.where_expr, query_info.internal_transformer
         )
@@ -775,8 +754,7 @@ class SQLTransformer(TransformerBaseClass):
         :param query_info:
         :return:
         """
-        frame = self.to_ibis_table(query_info)
-        return frame
+        return self._to_ibis_table(query_info)
 
     def union_all(
         self, expr1: TableExpr, expr2: TableExpr,
